<?php

session_start();
ini_set('display_errors', 'on');
error_reporting(E_ALL);

require_once 'PrizmPayClient/PrizmPayClient_API.php';
require_once 'PrizmPayClient/PrizmPayClient_PP.php';

$success = function ($message = 'Success') {
    header('Content-Type: application/json; charset=utf-8');
    $result = [
        'response' => ['code' => '200', 'message' => $message],
    ];
    exit(json_encode($result, JSON_PRETTY_PRINT));
};

$failure = function ($message = 'Failure') {
    header('Content-Type: application/json; charset=utf-8');
    $result = [
        'response' => ['code' => '400', 'message' => $message],
    ];
    exit(json_encode($result, JSON_PRETTY_PRINT));
};

try {

    /**
     * Testing URL: https://api-test.prizmmpay.com
     * Production URL: https://api.prizmmpay.com
     */
    $apiURL = 'https://api-test.prizmmpay.com';

    $merchant = [
        'merchantID' => 'YOUR_PRIZM_PAY_MERCHANT_ID',
        'secretKey'  => 'YOUR_PRIZM_PAY_MERCHANT_SECRET',
    ];

    //The encrypted payment token
    $paymentToken = filter_input(INPUT_POST, 'payment');

    $api = new PrizmPayClient_API($apiURL, $merchant['merchantID'], $merchant['secretKey']);

    //Decrypt the payment token to get its payment data
    $result = $api->decryptApplePayTokenByMerchant($paymentToken);

    if (!isset($result['status']) || $result['status'] !== 1 || empty($result['data'])) {
        throw new Exception(isset($result['message']) ? $result['message'] : 'Payment Token Decrypt Failure');
    }

    $decodedToken = $result['data'];

    //Convert the returned ISO currency number to ISO currency code
    if ($decodedToken['currency_code'] == 344) {
        $currencyCode = 'HKD';
    } else {
        throw new Exception('Invalid Currency Code');
    }

    $order               = [
        'merchantReferenceCode' => 'TEST-APPLE-PAY-' . time(),
        'amount'                => substr_replace(str_pad($decodedToken['transaction_amount'], 3, "0", STR_PAD_LEFT), '.', -2, 0),
        'currency'              => $currencyCode,
    ];
    $deviceFingerprintID = $merchant['merchantID'] . '_' . $order['merchantReferenceCode'] . '_' . session_id();

    $card = [
        'card_no'            => $decodedToken['application_primary_account_number'],
        'expiry_date_m'      => substr($decodedToken['application_expiration_date'], 2, 2),
        'expiry_date_y'      => substr($decodedToken['application_expiration_date'], 0, 2),
        'cvv'                => '123',
        'card_holder_name'   => 'Card Holder Name',
        'payment_cryptogram' => $decodedToken['online_payment_cryptogram'],
        'card_type'          => (substr($decodedToken['application_primary_account_number'], 0, 1) === '4') ? 'visa' : 'mastercard',
    ];

    $items = [
        ['name' => 'Test Item 1', 'quantity' => 1, 'unit_price' => 0.1],
    ];

    $pp = new PrizmPayClient_PP();
    $pp->setReferenceCode($order['merchantReferenceCode']);
    $pp->setInvoiceID($order['merchantReferenceCode']);
    $pp->setGrandTotal($order['amount'], $order['currency']);
    $pp->setDeviceFingerprintID($deviceFingerprintID);
    $pp->setBillingAddress('John', 'Doe', 'email@email.com', '99999999', 'Hong Kong', 'HK', 'HK', 'HK', 'Hong Kong', '00852');
    $pp->setShippingAddress('John', 'Doe', 'email@email.com', '99999999', 'Hong Kong', 'HK', 'HK', 'HK', 'Hong Kong', '00852');
    $pp->setCardHolderAddress('John', 'Doe', 'email@email.com', '99999999', 'Hong Kong', 'HK', 'HK', 'HK', 'Hong Kong', '00852');
    $pp->setCardInfo($card['card_no'], $card['card_holder_name'], $card['cvv'], $card['card_type'], $card['expiry_date_m'], $card['expiry_date_y']);
    $pp->setCardInfoValue('paymentCryptogram', $card['payment_cryptogram']);
    foreach ($items as $_item) {
        $pp->addItem($_item['name'], $_item['quantity'], $_item['unit_price']);
    }

    /**
     * Request authorize/sale API
     *
     * $api->authorizeByApplePay();
     * $api->saleByApplePay();
     */
    $responseData = $api->authorizeByApplePay($pp->data());

    $success();

} catch (Exception $e) {
    $failure($e->getMessage());
}
